@@ -1,12 +1,13 @@
+from __future__ import unicode_literals
 
-import os
-import re
 import py
-from .urlutil import DistURL
+from devpi_common.metadata import (sorted_sameproject_links,
+                                   get_latest_version, BasenameMeta)
+from devpi_common.validation import validate_metadata, normalize_name
+from devpi_common.types import ensure_unicode
+from devpi_common.archive import Archive
 from .vendor._description_utils import processDescription
-from .urlutil import sorted_by_version, get_latest_version
 from .auth import crypt_password, verify_password
-from .validation import validate_metadata, normalize_name
 
 import logging
 
@@ -112,22 +113,16 @@ class DB:
         return d
 
     def _get_user_and_index(self, user, index=None):
+        if not py.builtin._istext(user):
+            user = user.decode("utf8")
         if index is None:
             user = user.strip("/")
             user, index = user.split("/")
+        else:
+            if not py.builtin._istext(index):
+                index = index.decode("utf8")
         return user, index
 
-    def index_get(self, user, index=None):
-        user, index = self._get_user_and_index(user, index)
-        userconfig = self.keyfs.USER(user=user).get()
-        try:
-            indexconfig = userconfig["indexes"][index]
-        except KeyError:
-            return None
-        if "acl_upload" not in indexconfig:
-            indexconfig["acl_upload"] = [user]
-        return indexconfig
-
     def _normalize_bases(self, bases):
         # check and normalize base indices
         messages = []
@@ -168,10 +163,11 @@ class DB:
         with self.keyfs.USER(user=user).locked_update() as userconfig:
             indexes = userconfig.setdefault("indexes", {})
             assert index not in indexes, indexes[index]
-            indexes[index] = ixconfig = dict(
-                type=type, volatile=volatile, bases=bases,
-                uploadtrigger_jenkins=uploadtrigger_jenkins,
-                acl_upload=acl_upload)
+            indexes[index] = ixconfig = {
+                "type": type, "volatile": volatile, "bases": bases,
+                "uploadtrigger_jenkins": uploadtrigger_jenkins,
+                "acl_upload": acl_upload
+            }
             log.info("created index %s/%s: %s", user, index, ixconfig)
             return ixconfig
 
@@ -236,12 +232,22 @@ class ProjectInfo:
         self.name = name
         self.stage = stage
 
+    def __str__(self):
+        return "<ProjectInfo %s stage %s>" %(self.name, self.stage.name)
+
 
 class PrivateStage:
     metadata_keys = """
         name version summary home_page author author_email
         license description keywords platform classifiers download_url
     """.split()
+    # taken from distlib.metadata (6th October)
+    metadata_list_fields = ('platform', 'classifier', 'classifiers',
+               'obsoletes',
+               'requires', 'provides', 'obsoletes-Dist',
+               'provides-dist', 'requires-dist', 'requires-external',
+               'project-url', 'supported-platform', 'setup-requires-Dist',
+               'provides-extra', 'extension')
 
     def __init__(self, db, user, index, ixconfig):
         self.db = db
@@ -292,13 +298,15 @@ class PrivateStage:
     def get_project_info(self, name):
         """ return first matching project info object for the given name
         or None if no project exists. """
-        for stage, res in self.op_with_bases("get_project_info", name=name):
+        kwdict = {"name": name}
+        for stage, res in self.op_with_bases("get_project_info", **kwdict):
             if res is not None:
                 return res
 
     def get_project_info_perstage(self, name):
         """ return normalized name for the given name or None
         if no project exists. """
+        assert py.builtin._istext(name)
         names = self.getprojectnames_perstage()
         norm2name = dict([(normalize_name(x), x) for x in names])
         realname = norm2name.get(normalize_name(name), None)
@@ -310,7 +318,6 @@ class PrivateStage:
         errors. """
         validate_metadata(metadata)
         name = metadata["name"]
-        version = metadata["version"]
         # check if the project exists already under its normalized
         info = self.get_project_info(name)
         if info:
@@ -318,6 +325,8 @@ class PrivateStage:
         else:
             log.debug("project %r does not exist, good", name)
         if info is not None and info.name != name:
+            log.error("project %r has other name %r in stage %s" %(
+                      name, info.name, self.name))
             raise self.RegisterNameConflict(info)
         self._register_metadata(metadata)
 
@@ -342,11 +351,6 @@ class PrivateStage:
                 html = html.encode("utf8")
             key.set(html)
 
-    def project_add(self, name):
-        key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
-        with key.locked_update() as projectconfig:
-            pass
-
     def project_delete(self, name):
         key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
         key.delete()
@@ -371,7 +375,7 @@ class PrivateStage:
     def get_description(self, name, version):
         key = self.keyfs.RELDESCRIPTION(user=self.user, index=self.index,
             name=name, version=version)
-        return key.get()
+        return py.builtin._totext(key.get(), "utf-8")
 
     def get_description_versions(self, name):
         return self.keyfs.RELDESCRIPTION.listnames("version",
@@ -396,6 +400,7 @@ class PrivateStage:
         return key.get()
 
     def get_projectconfig(self, name):
+        assert py.builtin._istext(name)
         all_projectconfig = {}
         for stage, res in self.op_with_bases("get_projectconfig", name=name):
             if isinstance(res, int):
@@ -431,9 +436,7 @@ class PrivateStage:
                 if key not in basenames:
                     basenames.add(key)
                     all_links.append(entry)
-        all_links = sorted_by_version(all_links, attr="basename")
-        all_links.reverse()
-        return all_links
+        return sorted_sameproject_links(all_links)
 
     def getreleaselinks_perstage(self, projectname):
         projectconfig = self.get_projectconfig_perstage(projectname)
@@ -473,7 +476,9 @@ class PrivateStage:
         """ store_releasefile requires pre-existing release metadata. """
 
     def store_releasefile(self, filename, content, last_modified=None):
-        name, version = DistURL(filename).pkgname_and_version
+        filename = ensure_unicode(filename)
+        bmeta = BasenameMeta(filename)
+        name, version = bmeta.name, bmeta.version
         info = self.get_project_info(name)
         name = getattr(info, "name", name)
         if not self.get_metadata(name, version):
@@ -491,14 +496,26 @@ class PrivateStage:
             self.log_info("store_releasefile %s", entry.relpath)
             return entry
 
-    def store_doczip(self, name, content):
-        """ unzip doc content for the specified "name" project. """
-        assert content
-        key = self._doc_key(name)
+    def store_doczip(self, name, version, docfile):
+        """ store zip file and unzip doc content for the
+        specified "name" project. """
+        if not version:
+            version = self.get_metadata_latest(name)["version"]
+            log.info("store_doczip: derived version %s", version)
+        key = self.keyfs.PROJCONFIG(user=self.user, index=self.index, name=name)
+        with key.locked_update() as projectconfig:
+            verdata = projectconfig[version]
+            filename = "%s-%s.doc.zip" % (name, version)
+            entry = self.xom.filestore.store_file(self.user, self.index,
+                                filename, docfile)
+            verdata["+doczip"] = entry.relpath
+        # unpack
+        key = self._doc_key(name, version)
 
-        # XXX locking (unzipping could happen concurrently in theory)
+        # XXX locking? (unzipping could happen concurrently in theory)
         tempdir = self.keyfs.mkdtemp(name)
-        unzip_to_dir(content, tempdir)
+        with Archive(entry.filepath.open("rb")) as archive:
+            archive.extract(tempdir)
         keypath = key.filepath
         if keypath.check():
             old = keypath.new(basename="old-" + keypath.basename)
@@ -510,49 +527,19 @@ class PrivateStage:
             tempdir.move(keypath)
         return keypath
 
-    def get_doczip(self, name):
-        """ get zip file content of the current docs or None. """
-        key = self._doc_key(name)
-        basedir = key.filepath
-        if not basedir.check():
-            return
-        content = create_zipfile(basedir)
-        return content
+    def get_doczip(self, name, version):
+        """ get documentation zip as an open file
+        (or None if no docs exists). """
+        metadata = self.get_metadata(name, version)
+        if metadata:
+            doczip = metadata.get("+doczip")
+            if doczip:
+                entry = self.xom.filestore.getentry(doczip)
+                if entry:
+                    return entry.filepath.open("rb")
 
-    def _doc_key(self, name):
+    def _doc_key(self, name, version):
+        assert version
         return self.keyfs.STAGEDOCS(user=self.user, index=self.index,
-                                    name=name)
-
-
-def create_zipfile(basedir):
-    f = py.io.BytesIO()
-    zip = py.std.zipfile.ZipFile(f, "w")
-    _writezip(zip, basedir)
-    zip.close()
-    return f.getvalue()
-
-def _writezip(zip, basedir):
-    for p in basedir.visit():
-        if p.check(dir=1):
-            if not p.listdir():
-                path = p.relto(basedir) + "/"
-                zipinfo = py.std.zipfile.ZipInfo(path)
-                zip.writestr(zipinfo, "")
-        else:
-            path = p.relto(basedir)
-            zip.writestr(path, p.read("rb"))
-
-def unzip_to_dir(content, basedir):
-    unzipfile = py.std.zipfile.ZipFile(py.io.BytesIO(content))
-    members = unzipfile.namelist()
-    for name in members:
-        fpath = basedir.join(name, abs=True)
-        if not fpath.relto(basedir):
-            raise ValueError("invalid path name:" + name)
-        if name.endswith(os.sep) or name[-1] == "/":
-            fpath.ensure(dir=1)
-        else:
-            fpath.dirpath().ensure(dir=1)
-            with fpath.open("wb") as f:
-                f.write(unzipfile.read(name))
+                                    name=name, version=version)
 
