@@ -4,20 +4,20 @@ Implementation of the database layer for PyPI Package serving and
 testresult storage.
 
 """
-import json
-import re
+
+from __future__ import unicode_literals
+
 import py
 import sys
 import threading
 html = py.xml.html
 
-from .vendor._pip import HTMLPage
+from devpi_common.vendor._pip import HTMLPage
 
-from .types import propmapping
-from .urlutil import DistURL, joinpath
-from pkg_resources import Requirement
+from devpi_common.url import URL
+from devpi_common.metadata import is_archive_of_project, BasenameMeta
+from devpi_common.validation import normalize_name
 
-from .validation import normalize_name, safe_name
 from .db import ProjectInfo
 
 from logging import getLogger
@@ -46,15 +46,15 @@ class IndexParser:
     @property
     def releaselinks(self):
         """ return sorted releaselinks list """
-        l = list(self.basename2link.values())
-        l.sort(reverse=True)
-        return self.egglinks + l
+        l = sorted(map(BasenameMeta, self.basename2link.values()),
+                   reverse=True)
+        return self.egglinks + [x.obj for x in l]
 
     def parse_index(self, disturl, html, scrape=True):
         p = HTMLPage(html, disturl.url)
         seen = set()
         for link in p.links:
-            newurl = DistURL(link.url)
+            newurl = URL(link.url)
             if not newurl.is_valid_http_url():
                 continue
             eggfragment = newurl.eggfragment
@@ -69,20 +69,23 @@ class IndexParser:
                     log.debug("skip egg link %s (projectname: %s)",
                               newurl, self.projectname)
                 continue
-            if newurl.is_archive_of_project(self.projectname):
-                seen.add(newurl.url)
-                self._mergelink_ifbetter(newurl)
-                continue
+            if is_archive_of_project(newurl, self.projectname):
+                if not newurl.is_valid_http_url():
+                    log.warn("unparseable/unsupported url: %r", newurl)
+                else:
+                    seen.add(newurl.url)
+                    self._mergelink_ifbetter(newurl)
+                    continue
         if scrape:
             for link in p.rel_links():
                 if link.url not in seen:
-                    disturl = DistURL(link.url)
+                    disturl = URL(link.url)
                     if disturl.is_valid_http_url():
                         self.crawllinks.add(disturl)
 
 def parse_index(disturl, html, scrape=True):
-    if not isinstance(disturl, DistURL):
-        disturl = DistURL(disturl)
+    if not isinstance(disturl, URL):
+        disturl = URL(disturl)
     projectname = disturl.basename or disturl.parentbasename
     parser = IndexParser(projectname)
     parser.parse_index(disturl, html, scrape=scrape)
@@ -124,7 +127,7 @@ def perform_crawling(extdb, result, numthreads=10):
                 ct = response.headers.get("content-type", "").lower()
                 if ct.startswith("text/html"):
                     result.parse_index(
-                        DistURL(response.url), response.text, scrape=False)
+                        URL(response.url), response.text, scrape=False)
                     continue
             log.warn("crawlurl %s status %s", crawlurl, response)
 
@@ -159,7 +162,7 @@ def invalidate_on_version_change(basedir):
     verfile.write(ExtDB.VERSION)
 
 class ExtDB:
-    VERSION = "1"
+    VERSION = "2"
     name = "root/pypi"
     ixconfig = dict(bases=(), volatile=False, type="mirror")
 
@@ -254,14 +257,16 @@ class ExtDB:
             return releaselinks
         data = {}
         for link in releaselinks:
-            url = DistURL(link.url)
+            basename = link.basename
             if link.eggfragment:
                 version = "egg=" + link.eggfragment
             else:
-                _, version = url.pkgname_and_version
+                version = BasenameMeta(basename).version
             verdata = data.setdefault(version, {})
+            verdata["name"] = name
+            verdata["version"] = version
             files = verdata.setdefault("+files", {})
-            files[url.basename] = link.relpath
+            files[basename] = link.relpath
         return data
 
     get_projectconfig_perstage = get_projectconfig
@@ -285,6 +290,11 @@ class ExtDB:
             self.keyfs.PYPISERIALS.set(name2serials)
         else:
             log.info("reusing already cached name/serial list")
+        # normalize to unicode->serial mapping
+        for name in list(name2serials):
+            if not py.builtin._istext(name):
+                val = name2serials.pop(name)
+                name2serials[py.builtin._totext(name, "utf-8")] = val
         self.name2serials = name2serials
         # create a mapping of normalized name to real name
         self.normname2name = d = dict()
@@ -297,19 +307,16 @@ class ExtDB:
         """ set the current serial and fill normalization table
         if project does not exist.
         """
-        try:
-            current_serial = self.name2serials[name]
-        except KeyError:
+        if name in self.name2serials:
+            self.name2serials[name] = serial
+        else:
             self.name2serials[name] = serial
             n = normalize_name(name)
             if n != name:
                 self.normname2name[n] = name
-        else:
-            self.name2serials[name] = serial
 
     def spawned_pypichanges(self, proxy, proxysleep):
         log.info("changelog/update tasks starting")
-        keyfs = self.keyfs
         while 1:
             # get changes since the maximum serial we are aware of
             current_serial = max(itervalues(self.name2serials))
