@@ -1,20 +1,19 @@
-import execnet
+from __future__ import unicode_literals
 import sys
-import json
 import os
+import json
 import py
+import subprocess
 import logging
-from .validation import normalize_name
-from .types import cached_property
-from .urlutil import get_latest_version, Version
-from pkg_resources import parse_version
-
-from devpi_server.main import fatal
+from devpi_common.validation import normalize_name
+from devpi_common.metadata import Version, splitbasename
+from devpi_server.main import fatal, server_version
+from devpi_server.venv import create_server_venv
 import devpi_server
 
+
 def do_upgrade(xom):
     tw = py.io.TerminalWriter()
-    import subprocess
     serverdir = xom.config.serverdir
     exportdir = serverdir + "-export"
     if exportdir.check():
@@ -24,11 +23,27 @@ def do_upgrade(xom):
     script = sys.argv[0]
     def rel(p):
         return py.path.local().bestrelpath(p)
+    state_version = xom.get_state_version()
     tw.sep("-", "exporting to %s" % rel(exportdir))
-    subprocess.check_call([sys.executable, script,
-                           "--serverdir", str(serverdir),
-                           "--export", str(exportdir)])
+    if Version(state_version) > Version(server_version):
+        fatal("%s has state version %s which is newer than what we "
+              "can handle (%s)" %(
+                xom.config.serverdir, state_version, server_version))
+    elif Version(state_version) < Version(server_version):
+        tw.line("creating server venv %s ..." % state_version)
+        tmpdir = py.path.local.make_numbered_dir("devpi-upgrade")
+        venv = create_server_venv(tw, state_version, tmpdir)
+        tw.line("server venv %s created: %s" % (server_version, tmpdir))
+        venv.check_call(["devpi-server",
+                         "--serverdir", str(serverdir),
+                         "--export", str(exportdir)])
+    else:
+        #tw.sep("-", "creating venv" % rel(exportdir))
+        subprocess.check_call([sys.executable, script,
+                               "--serverdir", str(serverdir),
+                               "--export", str(exportdir)])
     tw.sep("-", "importing from %s" % rel(exportdir))
+    tw.line("importing into server version: %s" % server_version)
     subprocess.check_call([sys.executable, script,
                            "--serverdir", str(newdir),
                            "--import", str(exportdir)])
@@ -44,7 +59,7 @@ def do_upgrade(xom):
         serverdir.remove()
     tw.line("copying new serverstate to serverdir", bold=True)
     newdir.move(serverdir)
-    version = serverdir.join(".serverversion").read()
+    serverdir.join(".serverversion").read()
     tw.line("cleanup: removing exportdir: %s" % exportdir)
     tw.line("have fun serving the new state :)")
     exportdir.remove()
@@ -104,10 +119,10 @@ class Exporter:
     def dump_all(self, path):
         self.basepath = path
         self.export["dumpversion"] = self.DUMPVERSION
+        self.export["pythonversion"] = list(sys.version_info)
         self.export["devpi_server"] = devpi_server.__version__
         self.export["secret"] = self.config.secret
         self.compute_global_projectname_normalization()
-        users = self.export_users
         for username in self.db.user_list():
             userdir = path.join(username)
             data = self.db.user_get(username, credentials=True)
@@ -131,10 +146,14 @@ class Exporter:
             user = self.db.user_get(username)
             for indexname in user.get("indexes", []):
                 stage = self.db.getstage(username, indexname)
-                if stage.name == "root/pypi":
-                    continue
                 names = stage.getprojectnames_perstage()
                 for name in names:
+                    # pypi names take precedence for defining the realname
+                    if stage.name == "root/pypi":
+                        version = Version("999999.99999")
+                        version.realname = name
+                        norm2maxversion[normalize_name(name)] = version
+                        continue
                     config = stage.get_projectconfig_perstage(name)
                     if config:
                         maxver = None
@@ -173,7 +192,7 @@ class IndexDump:
         self.stage = stage
         self.basedir = basedir
         indexmeta = exporter.export_indexes[stage.name] = {}
-        indexmeta["projects"] = projects = {}
+        indexmeta["projects"] = {}
         indexmeta["indexconfig"] = stage.ixconfig
         indexmeta["files"] = []
         self.indexmeta = indexmeta
@@ -186,11 +205,13 @@ class IndexDump:
             projconfig.update(data)
             assert "+files" not in data
             for version, versiondata in data.items():
+                if not version:
+                    continue
                 versiondata["name"] = realname
                 self.dump_releasefiles(realname, versiondata)
-            content = self.stage.get_doczip(name)
-            if content:
-                self.dump_docfile(realname, content)
+                fil = self.stage.get_doczip(name, version)
+                if fil:
+                    self.dump_docfile(realname, version, fil)
         self.exporter.completed("index %r" % self.stage.name)
 
     def dump_releasefiles(self, projectname, versiondata):
@@ -228,10 +249,10 @@ class IndexDump:
                 self.exporter.completed("wrote attachment %s [%s]" %
                                  (p.relto(self.basedir), entry.basename))
 
-    def dump_docfile(self, projectname, content):
-        p = self.basedir.join(projectname + ".zip")
+    def dump_docfile(self, projectname, version, fil):
+        p = self.basedir.join("%s-%s.doc.zip" %(projectname, version))
         with p.open("wb") as f:
-            f.write(content)
+            f.write(fil.read())
         relpath = p.relto(self.exporter.basepath)
         self.add_filedesc("doczip", projectname, relpath)
 
@@ -245,7 +266,7 @@ class Importer:
 
     def read_json(self, path):
         self.tw.line("reading json: %s" %(path,))
-        return json.loads(path.read("rb"))
+        return json.loads(path.read())
 
     def warn(self, msg):
         self.tw.line(msg, red=True)
@@ -287,7 +308,6 @@ class Importer:
         # create projects and releasefiles for each index
         for stage in stages:
             assert stage.name != "root/pypi"
-            indexdir = self.import_rootdir.join("users", stage.name)
             import_index = self.import_indexes[stage.name]
             projects = import_index["projects"]
             #normalized = self.normalize_index_projects(projects)
@@ -308,7 +328,7 @@ class Importer:
     def import_filedesc(self, stage, filedesc):
         assert stage.ixconfig["type"] != "mirror"
         rel = filedesc["relpath"]
-        projectname = filedesc["projectname"]
+        #projectname = filedesc["projectname"]
         p = self.import_rootdir.join(rel)
         assert p.check(), p
         if filedesc["type"] == "releasefile":
@@ -319,7 +339,9 @@ class Importer:
             assert entry.size == mapping["size"]
             self.import_attachments(entry.md5)
         elif filedesc["type"] == "doczip":
-            stage.store_doczip(projectname, p.read("rb"))
+            basename = os.path.basename(rel)
+            name, version, suffix = splitbasename(basename)
+            stage.store_doczip(name, version, p.open("rb"))
         else:
             fatal("unknown file type: %s" % (type,))
 
