@@ -3,6 +3,7 @@
 a WSGI server to serve PyPI compatible indexes and a full
 recursive cache of pypi.python.org packages.
 """
+from __future__ import unicode_literals
 
 import os, sys
 import py
@@ -10,13 +11,13 @@ import threading
 from logging import getLogger
 log = getLogger(__name__)
 
-from .types import cached_property
+from devpi_common.types import cached_property
+from devpi_common.metadata import Version
+from devpi_common.request import new_requests_session
 from .config import parseoptions, configure_logging
-from .urlutil import Version
 
 import devpi_server
-from pkg_resources import parse_version
-from . import extpypi
+server_version = devpi_server.__version__
 
 PYPIURL_XMLRPC = "https://pypi.python.org/pypi/"
 
@@ -30,21 +31,11 @@ def check_compatible_version(xom):
     args = xom.config.args
     if args.upgrade_state or args.export:
         return
-    server_version = Version(devpi_server.__version__)
-    serverdir = xom.config.serverdir
-    versionfile = serverdir.join(".serverversion")
-    if not versionfile.check():
-        if not xom.db.is_empty():
-            fatal("serverdir %s is unversioned, assuming 1.0.\n"
-                  "Use --upgrade-state to migrate your data. " % serverdir)
-        # pypi mirror state is going to be invalidated anyway.
-    else:
-        state_version = Version(versionfile.read())
-        if server_version < state_version:
-            fatal("Incompatible state: server %s cannot run serverdir "
-                  "%s in %s. " %(server_version, serverdir, state_version))
-    versionfile.write(server_version.string)
-    log.info("setting new server version %s in serverdir" %(server_version))
+    state_version = xom.get_state_version()
+    if Version(server_version) != Version(state_version):
+        fatal("Incompatible state: server %s cannot run serverdir "
+              "%s created by %s. " %(server_version, xom.config.serverdir,
+                             state_version))
 
 def main(argv=None):
     """ devpi-server command line entry point. """
@@ -59,12 +50,12 @@ def _main(argv=None):
     if argv is None:
         argv = sys.argv
 
-    argv = map(str, argv)
+    argv = [str(x) for x in argv]
     config = parseoptions(argv)
     args = config.args
 
     if args.version:
-        print (devpi_server.__version__)
+        print(devpi_server.__version__)
         return
 
     # meta commmands (that will not instantiate server object in-process)
@@ -110,7 +101,7 @@ def bottle_run(xom):
     app = xom.create_app(immediatetasks=True,
                          catchall=not xom.config.args.debug)
     port = xom.config.args.port
-    log.info("devpi-server version: %s", devpi_server.__version__)
+    log.info("devpi-server version: %s", server_version)
     log.info("serverdir: %s" % xom.config.serverdir)
     hostaddr = "http://%s:%s" %(xom.config.args.host, xom.config.args.port)
     log.info("serving at url: %s", hostaddr)
@@ -130,7 +121,7 @@ def bottle_run(xom):
 def get_bottle_server(opt):
     if opt == "auto":
         try:
-            import eventlet
+            import eventlet  # noqa
         except ImportError:
             log.debug("could not import 'eventlet'")
             opt = "wsgiref"
@@ -143,19 +134,21 @@ def add_keys(keyfs):
     keyfs.USER = keyfs.addkey("{user}/.config", dict)
 
     # type pypimirror related data
-    keyfs.PYPILINKS = keyfs.addkey("root/pypi/links/{name}", dict)
-    keyfs.PYPIFILES = keyfs.addkey("root/pypi/f/{relpath}", file)
-    keyfs.PYPISERIALS = keyfs.addkey("root/pypi/serials", dict)
+    keyfs.PYPILINKS = keyfs.addkey("root/pypi/+links/{name}", dict)
+    keyfs.PYPISERIALS = keyfs.addkey("root/pypi/+serials", dict)
+    keyfs.PYPIFILE_NOMD5 = keyfs.addkey(
+        "{user}/{index}/+e/{relpath}", bytes)
 
     # type "stage" related
     keyfs.INDEXDIR = keyfs.addkey("{user}/{index}", "DIR")
     keyfs.PROJCONFIG = keyfs.addkey("{user}/{index}/{name}/.config", dict)
-    keyfs.STAGEFILE = keyfs.addkey(
-            "{user}/{index}/{proj}/{version}/{filename}", bytes)
-    keyfs.STAGEDOCS = keyfs.addkey("{user}/{index}/{name}/.doc", "DIR")
+    keyfs.STAGEFILE = keyfs.addkey("{user}/{index}/+f/{md5}/{filename}", bytes)
 
+    keyfs.STAGEDOCS = keyfs.addkey("{user}/{index}/{name}/{version}/+doc",
+                                   "DIR")
     keyfs.RELDESCRIPTION = keyfs.addkey(
             "{user}/{index}/{name}/{version}/description_html", bytes)
+
     keyfs.PATHENTRY = keyfs.addkey("{relpath}-meta", dict)
     keyfs.ATTACHMENT = keyfs.addkey("+attach/{md5}/{type}/{num}", bytes)
     keyfs.FILEPATH = keyfs.addkey("{relpath}", bytes)
@@ -174,6 +167,22 @@ class XOM:
 
         if httpget is not None:
             self.httpget = httpget
+        sdir = config.serverdir
+        if not (sdir.exists() and sdir.listdir()):
+            self.set_state_version(server_version)
+
+    def get_state_version(self):
+        versionfile = self.config.serverdir.join(".serverversion")
+        if not versionfile.exists():
+            fatal("serverdir %s is unversioned, please use depvi-server-1.1 "
+              "with the --upgrade-state option to upgrade from versions "
+              "prior to 1.1\n" % self.config.serverdir)
+        return versionfile.read()
+
+    def set_state_version(self, version):
+        versionfile = self.config.serverdir.join(".serverversion")
+        versionfile.dirpath().ensure(dir=1)
+        versionfile.write(version)
 
     def main(self):
         xom = self
@@ -254,7 +263,11 @@ class XOM:
     @cached_property
     def proxy(self):
         from devpi_server.extpypi import XMLProxy
-        from xmlrpclib import ServerProxy
+        try:
+            from xmlrpc.client import ServerProxy
+        except ImportError:
+            # PY2
+            from xmlrpclib import ServerProxy
         return XMLProxy(ServerProxy(PYPIURL_XMLRPC))
 
     @cached_property
@@ -266,11 +279,10 @@ class XOM:
 
     @cached_property
     def _httpsession(self):
-        import requests
-        return requests.session()
+        session = new_requests_session(agent=("server", server_version))
+        return session
 
     def httpget(self, url, allow_redirects, timeout=30):
-        from requests.exceptions import RequestException
         headers = {}
         USE_FRONT = self.config.args.bypass_cdn
         if USE_FRONT:
@@ -287,7 +299,7 @@ class XOM:
                 resp.url = resp.url.replace("https://front.python.org",
                                             "https://pypi.python.org")
             return resp
-        except RequestException:
+        except self._httpsession.RequestException:
             return FatalResponse(sys.exc_info())
 
     def create_app(self, catchall=True, immediatetasks=False):
@@ -352,4 +364,4 @@ def set_default_indexes(db):
         if "root" not in db.user_list():
             db.user_create("root", password="")
         db.index_create(PYPI, bases=(), type="mirror", volatile=False)
-        print ("set root/pypi default index")
+        print("set root/pypi default index")
