@@ -1,19 +1,21 @@
+from __future__ import unicode_literals
 import py
 from py.xml import html
-from devpi_server.types import lazydecorator, cached_property
+from devpi_common.types import lazydecorator, ensure_unicode
+from devpi_common.url import URL
+from devpi_common.metadata import get_pyversion_filetype
 import devpi_server
-from bottle import response, request, abort, redirect, HTTPError
-from bottle import BaseResponse, HTTPResponse, static_file
-import bottle
+from bottle import response, request, redirect, HTTPError
+from bottle import HTTPResponse, static_file
 import json
 import logging
-import inspect
-import requests
-from .validation import normalize_name, is_valid_archive_name
+from devpi_common.request import new_requests_session
+from devpi_common.validation import normalize_name, is_valid_archive_name
 
 from .auth import Auth
 from .config import render_string
-from . import urlutil
+
+server_version = devpi_server.__version__
 
 log = logging.getLogger(__name__)
 
@@ -28,7 +30,7 @@ def simple_html_body(title, bodytags, extrahead=""):
             extrahead,
         ),
         html.body(
-            html.h1(title),
+            html.h1(title), "\n",
             bodytags
         )
     )
@@ -41,8 +43,11 @@ def simple_html_body(title, bodytags, extrahead=""):
 
 API_VERSION = "1"
 
-meta_headers = {"X-DEVPI-API-VERSION": API_VERSION,
-                "X-DEVPI-SERVER-VERSION": devpi_server.__version__}
+# we use str() here so that python2.6 gets bytes, python3.3 gets string
+# so that wsgiref's parsing does not choke
+
+meta_headers = {str("X-DEVPI-API-VERSION"): API_VERSION,
+                str("X-DEVPI-SERVER-VERSION"): server_version}
 
 def abort(code, body):
     if "application/json" in request.headers.get("Accept", ""):
@@ -59,8 +64,8 @@ def abort_custom(code, msg):
 
 def abort_authenticate(msg="authentication required"):
     err = HTTPError(401, msg)
-    err.add_header('WWW-Authenticate', 'Basic realm="pypi"')
-    err.add_header('location', "/+login")
+    err.add_header(str('WWW-Authenticate'), 'Basic realm="pypi"')
+    err.add_header(str('location'), "/+login")
     raise err
 
 def apireturn(code, message=None, result=None, type=None):
@@ -73,13 +78,17 @@ def apireturn(code, message=None, result=None, type=None):
         d["message"] = message
     data = json.dumps(d, indent=2) + "\n"
     header = meta_headers.copy()
-    header["content-type"] = "application/json"
+    header[str("content-type")] = "application/json"
     raise HTTPResponse(body=data, status=code, header=header)
 
 def json_preferred():
     # XXX do proper "best" matching
     return "application/json" in request.headers.get("Accept", "")
 
+def html_preferred():
+    accept = request.headers.get("Accept", "")
+    return not accept or "text/html" in accept
+
 route = lazydecorator()
 
 class PyPIView:
@@ -132,8 +141,11 @@ class PyPIView:
         filestore = self.xom.filestore
         data = getjson(request)
         md5 = data["installpkg"]["md5"]
+        data = request.body.read()
+        if not py.builtin._istext(data):
+            data = data.decode("utf-8")
         num = filestore.add_attachment(md5=md5, type="toxresult",
-                                       data=request.body.read())
+                                       data=data)
         relpath = "/+tests/%s/%s/%s" %(md5, "toxresult", num)
         apireturn(200, type="testresultpath", result=relpath)
 
@@ -151,6 +163,7 @@ class PyPIView:
 
     #
     # index serving and upload
+    #
 
     #@route("/ext/pypi/simple<rest:re:.*>")  # deprecated
     #def extpypi_redirect(self, rest):
@@ -162,6 +175,7 @@ class PyPIView:
         # we only serve absolute links so we don't care about the route's slash
         abort_if_invalid_projectname(projectname)
         stage = self.getstage(user, index)
+        projectname = ensure_unicode(projectname)
         info = stage.get_project_info(projectname)
         if info and info.name != projectname:
             redirect("/%s/+simple/%s/" % (stage.name, info.name))
@@ -176,11 +190,11 @@ class PyPIView:
                 abort(502, "upstream server has internal error")
             if result < 0:
                 abort(502, "upstream server not reachable")
-
         links = []
         for entry in result:
             relpath = entry.relpath
             href = "/" + relpath
+            href = URL(request.path).relpath(href)
             if entry.eggfragment:
                 href += "#egg=%s" % entry.eggfragment
             elif entry.md5:
@@ -188,13 +202,14 @@ class PyPIView:
             links.extend([
                  "/".join(relpath.split("/", 2)[:2]) + " ",
                  html.a(entry.basename, href=href),
-                 html.br(),
+                 html.br(), "\n",
             ])
         return simple_html_body("%s: links for %s" % (stage.name, projectname),
-                                links).unicode()
+                                links).unicode(indent=2)
 
     @route("/<user>/<index>/+simple/")
     def simple_list_all(self, user, index):
+        encoding = "utf-8"
         log.info("starting +simple")
         stage = self.getstage(user, index)
         stage_results = []
@@ -205,24 +220,24 @@ class PyPIView:
 
         # at this point we are sure we can produce the data without
         # depending on remote networks
-        response.content_type = "text/html ; charset=utf-8"
+        response.content_type = "text/html ; charset=%s" % encoding
         title =  "%s: simple list (including inherited indices)" %(
                  stage.name)
-        yield "<html><head><title>%s</title></head><body><h1>%s</h1>" %(
-              title, title)
+        yield ("<html><head><title>%s</title></head><body><h1>%s</h1>" %(
+              title, title)).encode(encoding)
         all_names = set()
         for stage, names in stage_results:
             h2 = stage.name
             bases = getattr(stage, "ixconfig", {}).get("bases")
             if bases:
                 h2 += " (bases: %s)" % ",".join(bases)
-            yield ("<h2>" + h2 + "</h2>").encode("utf-8")
+            yield ("<h2>" + h2 + "</h2>").encode(encoding)
             for name in names:
                 if name not in all_names:
                     anchor = '<a href="%s/">%s</a><br/>' % (name, name)
-                    yield anchor.encode("utf-8")
+                    yield anchor.encode(encoding)
                     all_names.add(name)
-        yield "</body>"
+        yield "</body>".encode(encoding)
 
     @route("/<user>/<index>", method=["PUT", "PATCH"])
     def index_create_or_modify(self, user, index):
@@ -297,7 +312,7 @@ class PyPIView:
                       message="no release/files found for %s-%s" %(
                       name, version))
 
-        doczip = stage.get_doczip(name)
+        doczip = stage.get_doczip(name, version)
 
         # prepare metadata for submission
         metadata[":action"] = "submit"
@@ -318,7 +333,7 @@ class PyPIView:
             #results.append((r.status_code, "upload", entry.relpath))
             #apireturn(200, results=results, type="actionlog")
             if not target_stage.get_metadata(name, version):
-                self._register_metadata(target_stage, metadata)
+                self._register_metadata_dict(target_stage, metadata)
             results.append((200, "register", name, version,
                             "->", target_stage.name))
             for entry in matches:
@@ -329,7 +344,7 @@ class PyPIView:
                 results.append((res, "store_releasefile", entry.basename,
                                 "->", target_stage.name))
             if doczip:
-                target_stage.store_doczip(name, doczip)
+                target_stage.store_doczip(name, version, doczip)
                 results.append((200, "uploaded documentation", name,
                                 "->", target_stage.name))
             apireturn(200, result=results, type="actionlog")
@@ -339,7 +354,8 @@ class PyPIView:
             password = pushdata["password"]
             pypiauth = (username, password)
             log.info("registering %s-%s to %s", name, version, posturl)
-            r = requests.post(posturl, data=metadata, auth=pypiauth)
+            session = new_requests_session(agent=("server", server_version))
+            r = session.post(posturl, data=metadata, auth=pypiauth)
             log.debug("register returned: %s", r.status_code)
             ok_codes = (200, 201)
             results.append((r.status_code, "register", name, version))
@@ -348,20 +364,22 @@ class PyPIView:
                     file_metadata = metadata.copy()
                     file_metadata[":action"] = "file_upload"
                     basename = entry.basename
-                    pyver, filetype = urlutil.get_pyversion_filetype(basename)
+                    pyver, filetype = get_pyversion_filetype(basename)
                     file_metadata["filetype"] = filetype
                     file_metadata["pyversion"] = pyver
                     openfile = entry.FILE.filepath.open("rb")
-                    log.info("sending %s to %s", basename, posturl)
-                    r = requests.post(posturl, data=file_metadata,
+                    log.info("sending %s to %s, metadata %s",
+                             basename, posturl, file_metadata)
+                    r = session.post(posturl, data=file_metadata,
                           auth=pypiauth,
                           files={"content": (basename, openfile)})
                     log.debug("send finished, status: %s", r.status_code)
-                    results.append((r.status_code, "upload", entry.relpath))
+                    results.append((r.status_code, "upload", entry.relpath,
+                                    r.content))
                 if doczip:
                     doc_metadata = metadata.copy()
                     doc_metadata[":action"] = "doc_upload"
-                    r = requests.post(posturl, data=doc_metadata,
+                    r = session.post(posturl, data=doc_metadata,
                           auth=pypiauth,
                           files={"content": (name + ".zip", doczip)})
                     log.debug("send finished, status: %s", r.status_code)
@@ -383,45 +401,60 @@ class PyPIView:
         except KeyError:
             abort(400, ":action field not found")
         if action == "submit":
-            return self._register_metadata(stage, request.forms)
+            return self._register_metadata_form(stage, request.forms)
         elif action in ("doc_upload", "file_upload"):
             try:
                 content = request.files["content"]
             except KeyError:
                 abort(400, "content file field not found")
-            name = request.forms.get("name")
-            version = request.forms.get("version")
+            name = ensure_unicode(request.forms.get("name"))
+            version = ensure_unicode(request.forms.get("version"))
             info = stage.get_project_info(name)
             if not info:
                 abort(400, "no project named %r was ever registered" %(name))
             if action == "file_upload":
+                log.debug("metadata in form: %s", list(request.forms.items()))
                 abort_if_invalid_filename(name, content.filename)
                 if not stage.get_metadata(name, version):
-                    self._register_metadata(stage, request.forms)
+                    self._register_metadata_form(stage, request.forms)
                 res = stage.store_releasefile(content.filename, content.value)
                 if res == 409:
                     abort(409, "%s already exists in non-volatile index" %(
                          content.filename,))
-                jenkins_url = stage.ixconfig["uploadtrigger_jenkins"]
-                if jenkins_url:
-                    jenkins_url = jenkins_url.format(pkgname=name)
-                    if trigger_jenkins(stage, jenkins_url, name) == -1:
+                jenkinurl = stage.ixconfig["uploadtrigger_jenkins"]
+                if jenkinurl:
+                    jenkinurl = jenkinurl.format(pkgname=name)
+                    if trigger_jenkins(stage, jenkinurl, name) == -1:
                         abort_custom(200,
                             "OK, but couldn't trigger jenkins at %s" %
-                            (jenkins_url,))
+                            (jenkinurl,))
             else:
                 # docs have no version (XXX but they are tied to the latest)
                 if len(content.value) > MAXDOCZIPSIZE:
-                    abort(413, "zipfile too large")
-                stage.store_doczip(name, content.value)
+                    abort_custom(413, "zipfile size %d too large, max=%s"
+                                   % (len(content.value), MAXDOCZIPSIZE))
+                stage.store_doczip(name, version,
+                                   py.io.BytesIO(content.value))
         else:
             abort(400, "action %r not supported" % action)
         return ""
 
-    def _register_metadata(self, stage, form):
+    def _register_metadata_form(self, stage, form):
         metadata = {}
         for key in stage.metadata_keys:
-            metadata[key] = form.get(key, "")
+            if key.lower() in stage.metadata_list_fields:
+                val = [ensure_unicode(item)
+                        for item in form.getall(key) if item]
+            else:
+                val = getattr(form, key)  # returns unicode in bottle
+                if val == "UNKNOWN":
+                    val = ""
+                assert py.builtin._istext(val), val
+            metadata[key] = val
+
+        self._register_metadata_dict(stage, metadata)
+
+    def _register_metadata_dict(self, stage, metadata):
         try:
             stage.register_metadata(metadata)
         except stage.RegisterNameConflict as e:
@@ -439,12 +472,13 @@ class PyPIView:
     #
 
     # showing uploaded package documentation
-    @route("/<user>/<index>/<name>/+doc/<relpath:re:.*>",
+    @route("/<user>/<index>/<name>/<version>/+doc/<relpath:re:.*>",
            method="GET")
-    def doc_show(self, user, index, name, relpath):
+    def doc_show(self, user, index, name, version, relpath):
         if not relpath:
             redirect("index.html")
-        key = self.db.keyfs.STAGEDOCS(user=user, index=index, name=name)
+        stage = self.getstage(user, index)
+        key = stage._doc_key(name, version)
         if not key.filepath.check():
             abort(404, "no documentation available")
         return static_file(relpath, root=str(key.filepath))
@@ -452,31 +486,26 @@ class PyPIView:
     @route("/<user>/<index>/<name>")
     @route("/<user>/<index>/<name>/")
     def project_get(self, user, index, name):
+        #log.debug("HEADERS: %s", request.headers.items())
         stage = self.getstage(user, index)
+        name = ensure_unicode(name)
         info = stage.get_project_info(name)
-        if info and info.name != name:
-            redirect("/%s/%s/" % (stage.name, info.name))
+        real_name = info.name if info else name
+        if html_preferred():
+            # we need to redirect because the simple pages
+            # may return status codes != 200, causing
+            # pip to look at the full simple list at the parent url
+            # but we don't serve this list on /user/index/
+            redirect("/%s/+simple/%s/" % (stage.name, real_name))
+        if not json_preferred():
+            apireturn(415, "unsupported media type %s" %
+                      request.headers.items())
+        if not info:
+            apireturn(404, "project %r does not exist" % name)
+        if real_name != name:
+            redirect("/%s/%s/" % (stage.name, real_name))
         metadata = stage.get_projectconfig(name)
-        #if not metadata:
-        #    apireturn("404", "project %r does not exist" % name)
-        if json_preferred():
-            apireturn(200, type="projectconfig", result=metadata)
-        # html
-        body = []
-        for version in urlutil.sorted_by_version(metadata.keys()):
-            body.append(html.a(version, href=version + "/"))
-            body.append(html.br())
-        return simple_html_body("%s/%s: list of versions" % (stage.name,name),
-                                body).unicode(indent=2)
-
-    @route("/<user>/<index>/<name>", method="PUT")
-    def project_add(self, user, index, name):
-        self.require_user(user)
-        stage = self.getstage(user, index)
-        if stage.project_exists(name):
-            apireturn(409, "project %r exists" % name)
-        stage.project_add(name)
-        apireturn(201, "project %r created" % name)
+        apireturn(200, type="projectconfig", result=metadata)
 
     @route("/<user>/<index>/<name>", method="DELETE")
     @route("/<user>/<index>/<name>/", method="DELETE")
@@ -484,7 +513,7 @@ class PyPIView:
         self.require_user(user)
         stage = self.getstage(user, index)
         if stage.name == "root/pypi":
-            abort(405, "cannot delete on root/pypi index")
+            abort(405, "cannot delete root/pypi index")
         if not stage.project_exists(name):
             apireturn(404, "project %r does not exist" % name)
         if not stage.ixconfig["volatile"]:
@@ -497,6 +526,8 @@ class PyPIView:
     @route("/<user>/<index>/<name>/<version>/")
     def version_get(self, user, index, name, version):
         stage = self.getstage(user, index)
+        name = ensure_unicode(name)
+        version = ensure_unicode(version)
         metadata = stage.get_projectconfig(name)
         if not metadata:
             abort(404, "project %r does not exist" % name)
@@ -511,13 +542,14 @@ class PyPIView:
         for key, value in sorted(verdata.items()):
             if key == "description":
                 continue
+            if isinstance(value, list):
+                value = html.ul([html.li(x) for x in value])
             rows.append(html.tr(html.td(key), html.td(value)))
-        body = html.table(*rows)
         title = "%s/: %s-%s metadata and description" % (
                 stage.name, name, version)
 
         content = stage.get_description(name, version)
-        css = "https://pypi.python.org/styles/styles.css"
+        #css = "https://pypi.python.org/styles/styles.css"
         return simple_html_body(title,
             [html.table(*rows), py.xml.raw(content)],
             extrahead=
@@ -530,6 +562,8 @@ class PyPIView:
     @route("/<user>/<index>/<name>/<version>/", method="DELETE")
     def project_version_delete(self, user, index, name, version):
         stage = self.getstage(user, index)
+        name = ensure_unicode(name)
+        version = ensure_unicode(version)
         if stage.name == "root/pypi":
             abort(405, "cannot delete on root/pypi index")
         if not stage.ixconfig["volatile"]:
@@ -543,9 +577,9 @@ class PyPIView:
         stage.project_version_delete(name, version)
         apireturn(200, "project %r version %r deleted" % (name, version))
 
-    @route("/<user>/<index>/<name>/<version>/<relpath:re:.*>")
-    @route("/<user>/<index>/f/<relpath:re:.*>")
-    def pkgserv(self, user, index, name=None, version=None, relpath=None):
+    @route("/<user>/<index>/+e/<relpath:re:.*>")
+    @route("/<user>/<index>/+f/<relpath:re:.*>")
+    def pkgserv(self, user, index, relpath):
         relpath = request.path.strip("/")
         if "#" in relpath:   # XXX unclear how this can happen (it does)
             relpath = relpath.split("#", 1)[0]
@@ -594,23 +628,35 @@ class PyPIView:
                 bases = [html.h2("inherited bases"), bases]
         else:
             bases = []
-        latest_packages = html.ul()
+        latest_packages = html.table(
+            html.tr(html.td("info"), html.td("file"), html.td("docs")))
+
         for projectname in stage.getprojectnames_perstage():
             metadata = stage.get_metadata_latest(projectname)
-            name, ver = metadata["name"], metadata["version"]
-            dockey = stage._doc_key(name)
+            try:
+                name, ver = metadata["name"], metadata["version"]
+            except KeyError:
+                log.error("metadata for project %r empty: %s, skipping",
+                          projectname, metadata)
+                continue
+            dockey = stage._doc_key(name, ver)
             if dockey.exists():
-                docs = [" docs: ", html.a("%s-%s docs" %(name, ver),
-                                href="%s/+doc/index.html" %(name))]
+                docs = [html.a("%s-%s docs" %(name, ver),
+                        href="%s/%s/+doc/index.html" %(name, ver))]
             else:
                 docs = []
-            for basename, relpath in metadata["+files"].items():
-                latest_packages.append(html.li(
-                    html.a("%s-%s info page" % (name, ver),
-                           href="%s/%s/" % (name, ver)),
-                    " releasefiles: ",
-                    html.a(basename, href="/" + relpath),
-                    *docs
+            files = metadata.get("+files", {})
+            if not files:
+                log.warn("project %r version %r has no files", projectname,
+                         metadata.get("version"))
+            baseurl = URL(request.path)
+            for basename, relpath in files.items():
+                latest_packages.append(html.tr(
+                    html.td(html.a("%s-%s info page" % (name, ver),
+                           href="%s/%s/" % (name, ver))),
+                    html.td(html.a(basename,
+                                   href=baseurl.relpath("/" + relpath))),
+                    html.td(*docs),
                 ))
                 break  # could present more releasefiles
 
@@ -689,7 +735,7 @@ class PyPIView:
             apireturn(409, "user already exists")
         kvdict = getjson(request)
         if "password" in kvdict:  # and "email" in kvdict:
-            hash = self.db.user_create(user, **kvdict)
+            self.db.user_create(user, **kvdict)
             apireturn(201, type="userconfig", result=self.db.user_get(user))
         apireturn(400, "password needs to be set")
 
@@ -726,9 +772,15 @@ class PyPIView:
             d[user] = self.db.user_get(user)
         apireturn(200, type="list:userconfig", result=d)
 
+
 def getjson(request, allowed_keys=None):
     try:
-        dict = json.load(request.body)
+        # request.body is a StringIO on Py2 and a BytesIO on Py3. Convert
+        # it to a string, because json doesn't like bytes ...
+        content = request.body.read()
+        if not py.builtin._istext(content):
+            content = content.decode("utf-8")
+        dict = json.loads(content)
     except ValueError:
         abort(400, "Bad request: could not decode json")
     if allowed_keys is not None:
@@ -748,37 +800,38 @@ def get_outside_url(headers, outsideurl):
     log.debug("host header: %s", url)
     return url
 
-def trigger_jenkins(stage, jenkins_url, testspec):
+def trigger_jenkins(stage, jenkinurl, testspec):
     baseurl = get_outside_url(request.headers,
                               stage.xom.config.args.outside_url)
 
     source = render_string("devpibootstrap.py",
         INDEXURL=baseurl + stage.name,
         VIRTUALENVTARURL= (baseurl +
-            "root/pypi/f/https/pypi.python.org/packages/"
-            "source/v/virtualenv/virtualenv-1.10.1.tar.gz"),
+            "root/pypi/+f/3a04aa2b32c76c83725ed4d9918e362e/"
+            "virtualenv-1.10.1.tar.gz"),
         TESTSPEC=testspec,
         DEVPI_INSTALL_INDEX = baseurl + stage.name + "/+simple/"
     )
-    inputfile = py.io.BytesIO(source)
+    inputfile = py.io.BytesIO(source.encode("ascii"))
+    req = new_requests_session(agent=("server", server_version))
     try:
-        r = requests.post(jenkins_url, data={
+        r = req.post(jenkinurl, data={
                         "Submit": "Build",
                         "name": "jobscript.py",
                         "json": json.dumps(
                     {"parameter": {"name": "jobscript.py", "file": "file0"}}),
             },
                 files={"file0": ("file0", inputfile)})
-    except requests.exceptions.RequestException as e:
+    except req.RequestException:
         log.error("%s: failed to connect to jenkins at %s",
-                  testspec, jenkins_url)
+                  testspec, jenkinurl)
         return -1
 
     if 200 <= r.status_code < 300:
-        log.info("successfully triggered jenkins: %s", jenkins_url)
+        log.info("successfully triggered jenkins: %s", jenkinurl)
     else:
         log.error("%s: failed to trigger jenkins at %s", r.status_code,
-                  jenkins_url)
+                  jenkinurl)
         return -1
 
 def abort_if_invalid_filename(name, filename):
@@ -791,8 +844,11 @@ def abort_if_invalid_filename(name, filename):
 
 def abort_if_invalid_projectname(projectname):
     try:
-        projectname.decode("ascii")
-    except UnicodeDecodeError:
+        if isinstance(projectname, bytes):
+            projectname.decode("ascii")
+        else:
+            projectname.encode("ascii")
+    except (UnicodeEncodeError, UnicodeDecodeError):
         abort(400, "unicode project names not allowed")
 
 
@@ -823,3 +879,4 @@ def get_pure_metadata(somedict):
         if n[0] != "+":
             metadata[n] = v
     return metadata
+
