@@ -1,21 +1,20 @@
+from __future__ import unicode_literals
 
 import py
-import os
 import pytest
+from textwrap import dedent
 
-from devpi_server.urlutil import splitbasename
-from devpi_server.db import unzip_to_dir
+from devpi_common.metadata import splitbasename
+from devpi_common.archive import Archive, zip_dict
+from py.io import BytesIO
 
 
 @pytest.fixture(params=[(), ("root/pypi",)])
 def bases(request):
     return request.param
 
-def test_create_zipfile(tmpdir):
-    content = create_zipfile({"one": {"nested": "1"}, "two": {}})
-    tmpdir.join("hello.zip").write(content, "wb")
-
-def register_and_store(stage, basename, content="123", name=None):
+def register_and_store(stage, basename, content=b"123", name=None):
+    assert py.builtin._isbytes(content), content
     n, version = splitbasename(basename)[:2]
     if name is None:
         name = n
@@ -167,8 +166,8 @@ class TestStage:
         stage._reconfigure(bases=("root/pypi",))
         extdb.mock_simple("someproject",
             "<a href='someproject-1.0.zip' /a>")
-        register_and_store(stage, "someproject-1.0.zip", "123")
-        stage.store_releasefile("someproject-1.0.zip", "123")
+        register_and_store(stage, "someproject-1.0.zip", b"123")
+        stage.store_releasefile("someproject-1.0.zip", b"123")
         entries = stage.getreleaselinks("someproject")
         assert len(entries) == 1
         assert entries[0].relpath.endswith("someproject-1.0.zip")
@@ -176,13 +175,16 @@ class TestStage:
     def test_getreleaselinks_inheritance_shadow_egg(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
         extdb.mock_simple("py",
-        """<a href="http://bb.org/download/py.zip#egg=py-dev" />""")
-        register_and_store(stage, "py-1.0.tar.gz", "123")
+        """<a href="http://bb.org/download/py.zip#egg=py-dev" />
+           <a href="http://bb.org/download/master#egg=py-dev2" />
+        """)
+        register_and_store(stage, "py-1.0.tar.gz", b"123")
         entries = stage.getreleaselinks("py")
-        assert len(entries) == 2
-        e0, e1 = entries
-        assert e0.basename == "py.zip"
-        assert e1.basename == "py-1.0.tar.gz"
+        assert len(entries) == 3
+        e0, e1, e2 = entries
+        assert e0.basename == "py-1.0.tar.gz"
+        assert e1.basename == "py.zip"
+        assert e2.basename == "master"
 
     def test_inheritance_error(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
@@ -200,8 +202,7 @@ class TestStage:
         assert "someproject-1.0.zip" in projectconfig["1.0"]["+files"]
 
     def test_store_and_get_releasefile(self, stage, bases):
-        content = "123"
-        content2 = "1234"
+        content = b"123"
         entry = register_and_store(stage, "some-1.0.zip", content)
         entries = stage.getreleaselinks("some")
         assert len(entries) == 1
@@ -212,23 +213,23 @@ class TestStage:
 
     def test_store_releasefile_fails_if_not_registered(self, stage):
         with pytest.raises(stage.MissesRegistration):
-            stage.store_releasefile("someproject-1.0.zip", "123")
+            stage.store_releasefile("someproject-1.0.zip", b"123")
 
     def test_project_config_shadowed(self, extdb, stage):
         stage._reconfigure(bases=("root/pypi",))
         extdb.mock_simple("someproject",
             "<a href='someproject-1.0.zip' /a>")
-        content = "123"
-        entry = stage.store_releasefile("someproject-1.0.zip", content)
+        content = b"123"
+        stage.store_releasefile("someproject-1.0.zip", content)
         projectconfig = stage.get_projectconfig("someproject")
         files = projectconfig["1.0"]["+files"]
-        link = files.values()[0]
+        link = list(files.values())[0]
         assert link.endswith("someproject-1.0.zip")
         assert projectconfig["1.0"]["+shadowing"]
 
     def test_store_and_delete_project(self, stage, bases):
-        content = "123"
-        entry = register_and_store(stage, "some-1.0.zip", content)
+        content = b"123"
+        register_and_store(stage, "some-1.0.zip", content)
         pconfig = stage.get_projectconfig_perstage("some")
         assert pconfig["1.0"]
         stage.project_delete("some")
@@ -236,8 +237,8 @@ class TestStage:
         assert not pconfig
 
     def test_store_and_delete_release(self, stage, bases):
-        entry = register_and_store(stage, "some-1.0.zip")
-        entry = register_and_store(stage, "some-1.1.zip")
+        register_and_store(stage, "some-1.0.zip")
+        register_and_store(stage, "some-1.1.zip")
         pconfig = stage.get_projectconfig_perstage("some")
         assert pconfig["1.0"] and pconfig["1.1"]
         stage.project_version_delete("some", "1.0")
@@ -247,38 +248,35 @@ class TestStage:
         assert not stage.project_exists("some")
 
     def test_releasefile_sorting(self, stage, bases):
-        entry = register_and_store(stage, "some-1.1.zip")
-        entry = register_and_store(stage, "some-1.0.zip")
+        register_and_store(stage, "some-1.1.zip")
+        register_and_store(stage, "some-1.0.zip")
         entries = stage.getreleaselinks("some")
         assert len(entries) == 2
         assert entries[0].basename == "some-1.1.zip"
 
-    def test_storedoczipfile(self, stage, bases):
-        content = create_zipfile({"index.html": "<html/>",
-            "_static": {}, "_templ": {"x.css": ""}})
-        filepath = stage.store_doczip("pkg1", content)
-        assert filepath.join("index.html").check()
-        assert filepath.join("_static").check(dir=1)
-        assert filepath.join("_templ", "x.css").check(file=1)
-
     def test_getdoczip(self, stage, bases, tmpdir):
-        assert not stage.get_doczip("pkg1")
-        content = create_zipfile({"index.html": "<html/>",
+        assert not stage.get_doczip("pkg1", "version")
+        stage.register_metadata(dict(name="pkg1", version="1.0"))
+        content = zip_dict({"index.html": "<html/>",
             "_static": {}, "_templ": {"x.css": ""}})
-        filepath = stage.store_doczip("pkg1", content)
-        doczip_content = stage.get_doczip("pkg1")
-        assert doczip_content
-        unzip_to_dir(doczip_content, tmpdir)
+        stage.store_doczip("pkg1", "1.0", BytesIO(content))
+        doczip_file = stage.get_doczip("pkg1", "1.0")
+        assert doczip_file
+        with Archive(doczip_file) as archive:
+            archive.extract(tmpdir)
         assert tmpdir.join("index.html").read() == "<html/>"
         assert tmpdir.join("_static").check(dir=1)
         assert tmpdir.join("_templ", "x.css").check(file=1)
 
     def test_storedoczipfile(self, stage, bases):
-        content = create_zipfile({"index.html": "<html/>",
+        stage.register_metadata(dict(name="pkg1", version="1.0"))
+        content = zip_dict({"index.html": "<html/>",
             "_static": {}, "_templ": {"x.css": ""}})
-        filepath = stage.store_doczip("pkg1", content)
-        content = create_zipfile({"nothing": "hello"})
-        filepath = stage.store_doczip("pkg1", content)
+        filepath = stage.store_doczip("pkg1", "1.0", BytesIO(content))
+        assert filepath.join("index.html").exists()
+
+        content = zip_dict({"nothing": "hello"})
+        filepath = stage.store_doczip("pkg1", "1.0", BytesIO(content))
         assert filepath.join("nothing").check()
         assert not filepath.join("index.html").check()
         assert not filepath.join("_static").check()
@@ -286,8 +284,8 @@ class TestStage:
 
     def test_store_and_get_volatile(self, stage):
         stage._reconfigure(volatile=False)
-        content = "123"
-        content2 = "1234"
+        content = b"123"
+        content2 = b"1234"
         entry = register_and_store(stage, "some-1.0.zip", content)
         assert len(stage.getreleaselinks("some")) == 1
 
@@ -338,27 +336,32 @@ class TestStage:
         assert project.name == "Hello"
 
     def test_releasedata_description(self, stage):
-        source = py.std.textwrap.dedent("""\
+        source = py.builtin._totext(dedent("""\
             test the *world*
-        """)
+        """))
         assert stage.metadata_keys
         assert not stage.get_description("hello", "1.0")
         stage.register_metadata(dict(name="hello", version="1.0",
             description=source))
         html = stage.get_description("hello", "1.0")
-        assert html
+        assert py.builtin._istext(html)
         assert "test" in html and "world" in html
 
     def test_releasedata_description_versions(self, stage):
         stage.register_metadata(dict(name="hello", version="1.0",
-            description="hello"))
+            description=py.builtin._totext("hello")))
         stage.register_metadata(dict(name="hello", version="1.1",
-            description="hello"))
+            description=py.builtin._totext("hello")))
         ver = stage.get_description_versions("hello")
         assert set(ver) == set(["1.0", "1.1"])
 
 
 class TestUsers:
+
+    def test_secret(self, db):
+        db.keyfs.basedir.ensure(".something")
+        assert not db.user_get(".something")
+
     def test_create_and_validate(self, db):
         assert not db.user_exists("user")
         db.user_create("user", "password", email="some@email.com")
@@ -412,25 +415,3 @@ def test_setdefault_indexes(db):
     ixconfig = db.index_get("root/pypi")
     assert ixconfig["type"] == "mirror"
 
-def create_zipfile(contentdict):
-    f = py.io.BytesIO()
-    zip = py.std.zipfile.ZipFile(f, "w")
-    _writezip(zip, contentdict)
-    zip.close()
-    return f.getvalue()
-
-def _writezip(zip, contentdict, prefixes=()):
-    for name, val in contentdict.items():
-        if isinstance(val, dict):
-            newprefixes = prefixes + (name,)
-            if not val:
-                path = os.sep.join(newprefixes) + os.sep
-                zipinfo = py.std.zipfile.ZipInfo(path)
-                zip.writestr(zipinfo, "")
-            else:
-                _writezip(zip, val, newprefixes)
-        else:
-            path = os.sep.join(prefixes + (name,))
-            zip.writestr(path, val)
-
-
