@@ -1,12 +1,11 @@
 import mock
-import py
 import pytest
 
 from devpi_server.extpypi import *
-from devpi_server.main import FatalResponse, Fatal
+from devpi_server.main import Fatal
 
 class TestIndexParsing:
-    simplepy = DistURL("http://pypi.python.org/simple/py/")
+    simplepy = URL("http://pypi.python.org/simple/py/")
 
     def test_parse_index_simple(self):
         result = parse_index(self.simplepy,
@@ -23,7 +22,7 @@ class TestIndexParsing:
         assert link.url.endswith("/~user/py-1.4.12.zip#md5=12ab")
 
     def test_parse_index_simple_nocase(self):
-        simplepy = DistURL("http://pypi.python.org/simple/Py/")
+        simplepy = URL("http://pypi.python.org/simple/Py/")
         result = parse_index(simplepy,
             """<a href="../../pkg/py-1.4.12.zip#md5=12ab">qwe</a>
                <a href="../../pkg/PY-1.4.13.zip">qwe</a>
@@ -37,10 +36,14 @@ class TestIndexParsing:
         # it's not exactly clear how urlparse.uses_fragment
         # sometimes contains "svn" but it's good to check
         # that we are not sensitive to the issue.
-        import urlparse
+        try:
+            import urllib.parse as urlparse
+        except ImportError:
+            # PY2
+            import urlparse
         monkeypatch.setattr(urlparse, "uses_fragment",
                             urlparse.uses_fragment + ["svn"])
-        simplepy = DistURL("https://pypi.python.org/simple/zope.sqlalchemy/")
+        simplepy = URL("https://pypi.python.org/simple/zope.sqlalchemy/")
         result = parse_index(simplepy,
             '<a href="svn://svn.zope.org/repos/main/'
             'zope.sqlalchemy/trunk#egg=zope.sqlalchemy-dev" />'
@@ -50,7 +53,7 @@ class TestIndexParsing:
         #assert 0, (result.releaselinks, result.egglinks)
 
     def test_parse_index_normalized_name(self):
-        simplepy = DistURL("http://pypi.python.org/simple/ndg-httpsclient/")
+        simplepy = URL("http://pypi.python.org/simple/ndg-httpsclient/")
         result = parse_index(simplepy, """
                <a href="../../pkg/ndg_httpsclient-1.0.tar.gz" />
         """)
@@ -58,7 +61,7 @@ class TestIndexParsing:
         assert result.releaselinks[0].url.endswith("ndg_httpsclient-1.0.tar.gz")
 
     def test_parse_index_two_eggs_same_url(self):
-        simplepy = DistURL("http://pypi.python.org/simple/Py/")
+        simplepy = URL("http://pypi.python.org/simple/Py/")
         result = parse_index(simplepy,
             """<a href="../../pkg/pyzip#egg=py-dev">qwe2</a>
                <a href="../../pkg/pyzip#egg=py-dev">qwe</a>
@@ -129,13 +132,13 @@ class TestIndexParsing:
         assert link.basename == basename
 
     def test_parse_index_with_num_in_projectname(self):
-        simple = DistURL("http://pypi.python.org/simple/py-4chan/")
+        simple = URL("http://pypi.python.org/simple/py-4chan/")
         result = parse_index(simple, '<a href="pkg/py-4chan-1.0.zip"/>')
         assert len(result.releaselinks) == 1
         assert result.releaselinks[0].basename == "py-4chan-1.0.zip"
 
     def test_parse_index_unparseable_url(self):
-        simple = DistURL("http://pypi.python.org/simple/x123/")
+        simple = URL("http://pypi.python.org/simple/x123/")
         result = parse_index(simple, '<a href="http:" />')
         assert len(result.releaselinks) == 0
 
@@ -186,7 +189,7 @@ class TestIndexParsing:
         """)
         assert len(result.releaselinks) == 1
         assert len(result.crawllinks) == 2
-        result.parse_index(DistURL("http://pylib.org"), """
+        result.parse_index(URL("http://pylib.org"), """
                <a href="http://pylib.org/py-1.1-py27.egg" />
                <a href="http://pylib.org/other" rel="download" />
         """, scrape=False)
@@ -218,7 +221,7 @@ class TestIndexParsing:
         """)
         assert len(result.releaselinks) == 3
         assert len(result.crawllinks) == 2
-        result.parse_index(DistURL("http://pylib.org"), """
+        result.parse_index(URL("http://pylib.org"), """
                <a href="http://pylib.org/py-1.4.12.zip" />
                <a href="http://pylib.org/py-1.4.11.zip#md5=1111" />
                <a href="http://pylib.org/py-1.4.10.zip#md5=12ab" />
@@ -265,11 +268,14 @@ class TestExtPYPIDB:
 
 
     def test_getprojectconfig(self, extdb):
-        extdb.setextsimple("pytest", text='''
+        extdb.setextsimple("Pytest", text='''
             <a href="../../pkg/pytest-1.0.zip#md5=123" />''')
-        config = extdb.get_projectconfig("pytest")
+        config = extdb.get_projectconfig("Pytest")
         data = config["1.0"]
         assert data["+files"]
+        assert data["name"] == "Pytest"
+        assert data["version"] == "1.0"
+        assert extdb.get_project_info("pytest").name == "Pytest"
 
     def test_getdescription(self, extdb):
         extdb.setextsimple("pytest", text='''
@@ -460,7 +466,11 @@ class TestRefreshManager:
     @pytest.fixture(params=["protocol", "socket"])
     def raise_error(self, request):
         import socket
-        from xmlrpclib import ProtocolError
+        try:
+            from xmlrpc.client import ProtocolError
+        except ImportError:
+            # PY2
+            from xmlrpclib import ProtocolError
         if request.param == "protocol":
             exc = ProtocolError("http://pypi.python.org/pypi", 503, "", {})
         else:
@@ -472,7 +482,6 @@ class TestRefreshManager:
     def test_changelog_since_serial_nonetwork(self, extdb,
                     raise_error, monkeypatch, caplog):
         extdb.mock_simple("pytest", pypiserial=10)
-        from xmlrpclib import ServerProxy
         got = []
         def raise_xmlrpcish(since_int):
             got.append(since_int)
@@ -491,7 +500,7 @@ class TestRefreshManager:
         xmlproxy = mock.create_autospec(XMLProxy)
         xmlproxy.list_packages_with_serial.return_value = None
         with pytest.raises(Fatal):
-            xom = makexom(proxy=xmlproxy)
+            makexom(proxy=xmlproxy)
         #assert not xom.keyfs.PYPISERIALS.exists()
 
 
